@@ -9,8 +9,10 @@ from copy import deepcopy
 from lxml import etree
 import lxml.html
 import inspect
+import types
 import sys
 
+
 PY3k = sys.version_info >= (3,)
 
 if PY3k:
@@ -31,6 +33,26 @@ def func_code(f):
     return f.__code__ if PY3k else f.func_code
 
 
+def with_camel_case_alias(func):
+    """decorator for methods who required a camelcase alias"""
+    _camel_case_aliases.add(func.__name__)
+    return func
+_camel_case_aliases = set()
+
+
+def build_camel_case_aliases(PyQuery):
+    """add camelcase aliases to PyQuery"""
+    for alias in _camel_case_aliases:
+        parts = list(alias.split('_'))
+        name = parts[0] + ''.join([p.title() for p in parts[1:]])
+        func = getattr(PyQuery, alias)
+        f = types.FunctionType(func_code(func), func_globals(func),
+                               name, inspect.getargspec(func).defaults)
+        f.__doc__ = (
+            'Alias for :func:`~pyquery.pyquery.PyQuery.%s`') % func.__name__
+        setattr(PyQuery, name, f.__get__(None, PyQuery))
+
+
 def fromstring(context, parser=None, custom_parser=None):
     """use html parser if we don't have clean xml
     """
@@ -43,6 +65,8 @@ def fromstring(context, parser=None, custom_parser=None):
             try:
                 result = getattr(etree, meth)(context)
             except etree.XMLSyntaxError:
+                if hasattr(context, 'seek'):
+                    context.seek(0)
                 result = getattr(lxml.html, meth)(context)
             if isinstance(result, etree._ElementTree):
                 return [result.getroot()]
@@ -52,13 +76,16 @@ def fromstring(context, parser=None, custom_parser=None):
             custom_parser = getattr(etree, meth)
         elif parser == 'html':
             custom_parser = getattr(lxml.html, meth)
+        elif parser == 'html5':
+            from lxml.html import html5parser
+            custom_parser = getattr(html5parser, meth)
         elif parser == 'soup':
-            from  lxml.html import soupparser
+            from lxml.html import soupparser
             custom_parser = getattr(soupparser, meth)
         elif parser == 'html_fragments':
             custom_parser = lxml.html.fragments_fromstring
         else:
-            ValueError('No such parser: "%s"' % parser)
+            raise ValueError('No such parser: "%s"' % parser)
 
     result = custom_parser(context)
     if type(result) is list:
@@ -120,18 +147,19 @@ class FlexibleElement(object):
 class PyQuery(list):
     """The main class
     """
+
+    _translator_class = JQueryTranslator
+
     def __init__(self, *args, **kwargs):
         html = None
         elements = []
         self._base_url = None
-        self.parser = kwargs.get('parser', None)
-        if 'parser' in kwargs:
-            del kwargs['parser']
+        self.parser = kwargs.pop('parser', None)
 
-        if len(args) >= 1 and \
-           (not PY3k and isinstance(args[0], basestring) or \
-           (PY3k and isinstance(args[0], str))) and \
-           args[0].split('://', 1)[0] in ('http', 'https'):
+        if (len(args) >= 1 and
+                (not PY3k and isinstance(args[0], basestring) or
+                (PY3k and isinstance(args[0], str))) and
+                args[0].split('://', 1)[0] in ('http', 'https')):
             kwargs['url'] = args[0]
             if len(args) >= 2:
                 kwargs['data'] = args[1]
@@ -145,15 +173,13 @@ class PyQuery(list):
         if 'css_translator' in kwargs:
             self._translator = kwargs.pop('css_translator')
         elif self.parser in ('xml',):
-            self._translator = JQueryTranslator(xhtml=True)
+            self._translator = self._translator_class(xhtml=True)
         elif self._parent is not no_default:
             self._translator = self._parent._translator
         else:
-            self._translator = JQueryTranslator(xhtml=False)
+            self._translator = self._translator_class(xhtml=False)
 
-        namespaces = kwargs.get('namespaces', {})
-        if 'namespaces' in kwargs:
-            del kwargs['namespaces']
+        namespaces = kwargs.pop('namespaces', {})
 
         if kwargs:
             # specific case to get the dom
@@ -191,8 +217,8 @@ class PyQuery(list):
             elif length == 2:
                 selector, context = args
             else:
-                raise ValueError("You can't do that." +\
-                        " Please, provide arguments")
+                raise ValueError(
+                    "You can't do that. Please, provide arguments")
 
             # get context
             if isinstance(context, basestring):
@@ -230,9 +256,10 @@ class PyQuery(list):
             raise ValueError('You must provide at least a selector')
         if args[0] == '':
             return self.__class__([])
-        if len(args) == 1 and \
-           isinstance(args[0], str) and \
-           not args[0].startswith('<'):
+        if (len(args) == 1 and
+                (not PY3k and isinstance(args[0], basestring) or
+                (PY3k and isinstance(args[0], str))) and
+                not args[0].startswith('<')):
             args += (self,)
         result = self.__class__(*args, parent=self, **kwargs)
         return result
@@ -259,10 +286,15 @@ class PyQuery(list):
             ['foo', 'bar']
             >>> [i.text() for i in d('span').items()]
             ['foo', 'bar']
+            >>> list(d.items('a')) == list(d('a').items())
+            True
         """
-        elems = selector and self(selector) or self
+        if selector:
+            elems = self(selector) or []
+        else:
+            elems = self
         for elem in elems:
-            yield self.__class__(elem)
+            yield self.__class__(elem, **dict(parent=self))
 
     def xhtml_to_html(self):
         """Remove xhtml namespace:
@@ -271,7 +303,7 @@ class PyQuery(list):
             ...         '<html xmlns="http://www.w3.org/1999/xhtml"></html>')
             >>> doc
             [<{http://www.w3.org/1999/xhtml}html>]
-            >>> doc.remove_namespaces()
+            >>> doc.xhtml_to_html()
             [<html>]
         """
         try:
@@ -317,8 +349,8 @@ class PyQuery(list):
 
     def __unicode__(self):
         """xml representation of current nodes"""
-        return unicode('').join([etree.tostring(e, encoding=unicode) \
-                                                        for e in self])
+        return unicode('').join([etree.tostring(e, encoding=unicode)
+                                 for e in self])
 
     def __html__(self):
         """html representation of current nodes::
@@ -329,8 +361,8 @@ class PyQuery(list):
             <script><![[CDATA[ ]></script>
 
         """
-        return unicode('').join([lxml.html.tostring(e, encoding=unicode) \
-                                                            for e in self])
+        return unicode('').join([lxml.html.tostring(e, encoding=unicode)
+                                 for e in self])
 
     def __repr__(self):
         r = []
@@ -396,19 +428,19 @@ class PyQuery(list):
 
     def parent(self, selector=None):
         return self._filter_only(
-                   selector,
-                   [e.getparent() for e in self if e.getparent() is not None],
-                   unique=True)
+            selector,
+            [e.getparent() for e in self if e.getparent() is not None],
+            unique=True)
 
     def prev(self, selector=None):
         return self._filter_only(
-               selector,
-               [e.getprevious() for e in self if e.getprevious() is not None])
+            selector,
+            [e.getprevious() for e in self if e.getprevious() is not None])
 
     def next(self, selector=None):
         return self._filter_only(
-               selector,
-               [e.getnext() for e in self if e.getnext() is not None])
+            selector,
+            [e.getnext() for e in self if e.getnext() is not None])
 
     def _traverse(self, method):
         for e in self:
@@ -428,29 +460,35 @@ class PyQuery(list):
             for j in this_list:
                 yield j
 
-    def _nextAll(self):
+    def _next_all(self):
         return [e for e in self._traverse('getnext')]
 
-    def nextAll(self, selector=None):
+    @with_camel_case_alias
+    def next_all(self, selector=None):
         """
         >>> h = '<span><p class="hello">Hi</p><p>Bye</p><img scr=""/></span>'
         >>> d = PyQuery(h)
+        >>> d('p:last').next_all()
+        [<img>]
         >>> d('p:last').nextAll()
         [<img>]
         """
-        return self._filter_only(selector, self._nextAll())
+        return self._filter_only(selector, self._next_all())
 
-    def _prevAll(self):
+    def _prev_all(self):
         return [e for e in self._traverse('getprevious')]
 
-    def prevAll(self, selector=None):
+    @with_camel_case_alias
+    def prev_all(self, selector=None):
         """
         >>> h = '<span><p class="hello">Hi</p><p>Bye</p><img scr=""/></span>'
         >>> d = PyQuery(h)
+        >>> d('p:last').prev_all()
+        [<p.hello>]
         >>> d('p:last').prevAll()
         [<p.hello>]
         """
-        return self._filter_only(selector, self._prevAll(), reverse=True)
+        return self._filter_only(selector, self._prev_all(), reverse=True)
 
     def siblings(self, selector=None):
         """
@@ -462,7 +500,7 @@ class PyQuery(list):
          [<img>]
 
         """
-        return self._filter_only(selector, self._prevAll() + self._nextAll())
+        return self._filter_only(selector, self._prev_all() + self._next_all())
 
     def parents(self, selector=None):
         """
@@ -475,10 +513,10 @@ class PyQuery(list):
         []
         """
         return self._filter_only(
-                selector,
-                [e for e in self._traverse_parent_topdown()],
-                unique=True
-            )
+            selector,
+            [e for e in self._traverse_parent_topdown()],
+            unique=True
+        )
 
     def children(self, selector=None):
         """Filter elements that are direct children of self using optional
@@ -509,8 +547,8 @@ class PyQuery(list):
         """
         result = []
         for current in self:
-            while current is not None and \
-                  not self.__class__(current).is_(selector):
+            while (current is not None and
+                    not self.__class__(current).is_(selector)):
                 current = current.getparent()
             if current is not None:
                 result.append(current)
@@ -576,16 +614,19 @@ class PyQuery(list):
         """Returns True if selector matches at least one current element, else
         False:
 
-            >>> d = PyQuery('<p class="hello">Hi</p><p>Bye</p><div></div>')
+            >>> d = PyQuery('<p class="hello"><span>Hi</span></p><p>Bye</p>')
             >>> d('p').eq(0).is_('.hello')
             True
 
+            >>> d('p').eq(0).is_('span')
+            False
+
             >>> d('p').eq(1).is_('.hello')
             False
 
         ..
         """
-        return bool(self.__class__(selector, self))
+        return bool(self._filter_only(selector, self))
 
     def find(self, selector):
         """Find elements using selector traversing down from self:
@@ -598,8 +639,8 @@ class PyQuery(list):
             [<em>]
         """
         xpath = self._css_to_xpath(selector)
-        results = [child.xpath(xpath) for tag in self \
-                        for child in tag.getchildren()]
+        results = [child.xpath(xpath) for tag in self
+                   for child in tag.getchildren()]
         # Flatten the results
         elements = []
         for r in results:
@@ -629,7 +670,7 @@ class PyQuery(list):
         try:
             for i, element in enumerate(self):
                 func_globals(func)['this'] = element
-                if callback(func, i, element) == False:
+                if callback(func, i, element) is False:
                     break
         finally:
             f_globals = func_globals(func)
@@ -720,26 +761,32 @@ class PyQuery(list):
         elif value is no_default:
             return self[0].get(attr)
         elif value is None or value == '':
-            return self.removeAttr(attr)
+            return self.remove_attr(attr)
         else:
             for tag in self:
                 tag.set(attr, value)
         return self
 
-    def removeAttr(self, name):
+    @with_camel_case_alias
+    def remove_attr(self, name):
         """Remove an attribute::
 
             >>> d = PyQuery('<div id="myid"></div>')
+            >>> d.remove_attr('id')
+            [<div>]
             >>> d.removeAttr('id')
             [<div>]
 
         ..
         """
         for tag in self:
-            del tag.attrib[name]
+            try:
+                del tag.attrib[name]
+            except KeyError:
+                pass
         return self
 
-    attr = FlexibleElement(pget=attr, pdel=removeAttr)
+    attr = FlexibleElement(pget=attr, pdel=remove_attr)
 
     #######
     # CSS #
@@ -754,10 +801,13 @@ class PyQuery(list):
         """
         return self.attr('width', value)
 
-    def hasClass(self, name):
+    @with_camel_case_alias
+    def has_class(self, name):
         """Return True if element has class::
 
             >>> d = PyQuery('<div class="myclass"></div>')
+            >>> d.has_class('myclass')
+            True
             >>> d.hasClass('myclass')
             True
 
@@ -765,10 +815,13 @@ class PyQuery(list):
         """
         return self.is_('.%s' % name)
 
-    def addClass(self, value):
+    @with_camel_case_alias
+    def add_class(self, value):
         """Add a css class to elements::
 
             >>> d = PyQuery('<div></div>')
+            >>> d.add_class('myclass')
+            [<div.myclass>]
             >>> d.addClass('myclass')
             [<div.myclass>]
 
@@ -781,10 +834,13 @@ class PyQuery(list):
             tag.set('class', ' '.join(classes))
         return self
 
-    def removeClass(self, value):
+    @with_camel_case_alias
+    def remove_class(self, value):
         """Remove a css class to elements::
 
             >>> d = PyQuery('<div class="myclass"></div>')
+            >>> d.remove_class('myclass')
+            [<div>]
             >>> d.removeClass('myclass')
             [<div>]
 
@@ -795,15 +851,22 @@ class PyQuery(list):
             classes = set((tag.get('class') or '').split())
             classes.difference_update(values)
             classes.difference_update([''])
-            tag.set('class', ' '.join(classes))
+            classes = ' '.join(classes)
+            if classes.strip():
+                tag.set('class', classes)
+            elif tag.get('class'):
+                tag.set('class', classes)
         return self
 
-    def toggleClass(self, value):
+    @with_camel_case_alias
+    def toggle_class(self, value):
         """Toggle a css class to elements
 
             >>> d = PyQuery('<div></div>')
-            >>> d.toggleClass('myclass')
+            >>> d.toggle_class('myclass')
             [<div.myclass>]
+            >>> d.toggleClass('myclass')
+            [<div>]
 
         """
         for tag in self:
@@ -846,10 +909,11 @@ class PyQuery(list):
         elif isinstance(value, basestring):
             attr = attr.replace('_', '-')
             for tag in self:
-                current = [el.strip()
-                           for el in (tag.get('style') or '').split(';')
-                           if el.strip()
-                              and not el.split(':')[0].strip() == attr.strip()]
+                current = [
+                    el.strip()
+                    for el in (tag.get('style') or '').split(';')
+                    if (el.strip() and
+                        not el.split(':')[0].strip() == attr.strip())]
                 current.append('%s: %s' % (attr, value))
                 tag.set('style', '; '.join(current))
         return self
@@ -929,8 +993,8 @@ class PyQuery(list):
             html = tag.text or ''
             if 'encoding' not in kwargs:
                 kwargs['encoding'] = unicode
-            html += unicode('').join([etree.tostring(e, **kwargs) \
-                                                  for e in children])
+            html += unicode('').join([etree.tostring(e, **kwargs)
+                                      for e in children])
             return html
         else:
             if isinstance(value, self.__class__):
@@ -946,8 +1010,8 @@ class PyQuery(list):
                 for child in tag.getchildren():
                     tag.remove(child)
                 root = fromstring(
-                            unicode('<root>') + new_html + unicode('</root>'),
-                            self.parser)[0]
+                    unicode('<root>') + new_html + unicode('</root>'),
+                    self.parser)[0]
                 children = root.getchildren()
                 if children:
                     tag.extend(children)
@@ -955,17 +1019,20 @@ class PyQuery(list):
                 tag.tail = root.tail
         return self
 
-    def outerHtml(self):
+    @with_camel_case_alias
+    def outer_html(self):
         """Get the html representation of the first selected element::
 
             >>> d = PyQuery('<div><span class="red">toto</span> rocks</div>')
             >>> print(d('span'))
             <span class="red">toto</span> rocks
+            >>> print(d('span').outer_html())
+            <span class="red">toto</span>
             >>> print(d('span').outerHtml())
             <span class="red">toto</span>
 
             >>> S = PyQuery('<p>Only <b>me</b> & myself</p>')
-            >>> print(S('b').outerHtml())
+            >>> print(S('b').outer_html())
             <b>me</b>
 
         ..
@@ -999,7 +1066,7 @@ class PyQuery(list):
 
         if value is no_default:
             if not self:
-                return None
+                return ''
 
             text = []
 
@@ -1026,7 +1093,7 @@ class PyQuery(list):
     ################
 
     def _get_root(self, value):
-        if  isinstance(value, basestring):
+        if isinstance(value, basestring):
             root = fromstring(unicode('<root>') + value + unicode('</root>'),
                               self.parser)[0]
         elif isinstance(value, etree._Element):
@@ -1035,7 +1102,7 @@ class PyQuery(list):
             root = value
         else:
             raise TypeError(
-            'Value must be string, PyQuery or Element. Got %r' % value)
+                'Value must be string, PyQuery or Element. Got %r' % value)
         if hasattr(root, 'text') and isinstance(root.text, basestring):
             root_text = root.text
         else:
@@ -1062,7 +1129,8 @@ class PyQuery(list):
             root = tag[-len(root):]
         return self
 
-    def appendTo(self, value):
+    @with_camel_case_alias
+    def append_to(self, value):
         """append nodes to value
         """
         value.append(self)
@@ -1086,7 +1154,8 @@ class PyQuery(list):
             root = tag[:len(root)]
         return self
 
-    def prependTo(self, value):
+    @with_camel_case_alias
+    def prepend_to(self, value):
         """prepend nodes to value
         """
         value.prepend(self)
@@ -1108,7 +1177,8 @@ class PyQuery(list):
             root = parent[index:len(root)]
         return self
 
-    def insertAfter(self, value):
+    @with_camel_case_alias
+    def insert_after(self, value):
         """insert nodes after value
         """
         value.after(self)
@@ -1120,7 +1190,7 @@ class PyQuery(list):
         root, root_text = self._get_root(value)
         for i, tag in enumerate(self):
             previous = tag.getprevious()
-            if previous != None:
+            if previous is not None:
                 if not previous.tail:
                     previous.tail = ''
                 previous.tail += root_text
@@ -1137,7 +1207,8 @@ class PyQuery(list):
             root = parent[index:len(root)]
         return self
 
-    def insertBefore(self, value):
+    @with_camel_case_alias
+    def insert_before(self, value):
         """insert nodes before value
         """
         value.before(self)
@@ -1178,10 +1249,15 @@ class PyQuery(list):
         self[:] = nodes
         return self
 
-    def wrapAll(self, value):
+    @with_camel_case_alias
+    def wrap_all(self, value):
         """Wrap all the elements in the matched set into a single wrapper
         element::
 
+            >>> d = PyQuery('<div><span>Hey</span><span>you !</span></div>')
+            >>> print(d('span').wrap_all('<div id="wrapper"></div>'))
+            <div id="wrapper"><span>Hey</span><span>you !</span></div>
+
             >>> d = PyQuery('<div><span>Hey</span><span>you !</span></div>')
             >>> print(d('span').wrapAll('<div id="wrapper"></div>'))
             <div id="wrapper"><span>Hey</span><span>you !</span></div>
@@ -1222,13 +1298,25 @@ class PyQuery(list):
         self[:] = [wrapper]
         return self
 
-    def replaceWith(self, value):
-        """replace nodes by value
+    @with_camel_case_alias
+    def replace_with(self, value):
+        """replace nodes by value::
+
+            >>> doc = PyQuery("<html><div /></html>")
+            >>> node = PyQuery("<span />")
+            >>> child = doc.find('div')
+            >>> child.replace_with(node)
+            [<div>]
+            >>> print(doc)
+            <html><span/></html>
+
         """
+        if isinstance(value, PyQuery):
+            value = str(value)
         if hasattr(value, '__call__'):
             for i, element in enumerate(self):
                 self.__class__(element).before(
-                        value(i, element) + (element.tail or ''))
+                    value(i, element) + (element.tail or ''))
                 parent = element.getparent()
                 parent.remove(element)
         else:
@@ -1238,13 +1326,14 @@ class PyQuery(list):
                 parent.remove(tag)
         return self
 
-    def replaceAll(self, expr):
+    @with_camel_case_alias
+    def replace_all(self, expr):
         """replace nodes by expr
         """
         if self._parent is no_default:
             raise ValueError(
-                    'replaceAll can only be used with an object with parent')
-        self._parent(expr).replaceWith(self)
+                'replaceAll can only be used with an object with parent')
+        self._parent(expr).replace_with(self)
         return self
 
     def clone(self):
@@ -1304,9 +1393,9 @@ class PyQuery(list):
 
         """
         def __setattr__(self, name, func):
-            def fn(self, *args):
+            def fn(self, *args, **kwargs):
                 func_globals(func)['this'] = self
-                return func(*args)
+                return func(*args, **kwargs)
             fn.__name__ = name
             setattr(PyQuery, name, fn)
     fn = Fn()
@@ -1330,8 +1419,16 @@ class PyQuery(list):
         if base_url is None:
             base_url = self.base_url
             if base_url is None:
-                raise ValueError('You need a base URL to make your links'
-                 'absolute. It can be provided by the base_url parameter.')
+                raise ValueError((
+                    'You need a base URL to make your links'
+                    'absolute. It can be provided by the base_url parameter.'))
 
-        self('a').each(lambda: self(this).attr('href', urljoin(base_url, self(this).attr('href')))) # NOQA
+        def repl(i, e):
+            return self(e).attr(
+                'href',
+                urljoin(base_url, self(e).attr('href')))
+
+        self('a').each(repl)
         return self
+
+build_camel_case_aliases(PyQuery)
